@@ -13,7 +13,7 @@ import os
 import struct
 import sys
 import time
-from io import BytesIO
+from io import BytesIO, TextIOWrapper
 from unittest import skipIf
 
 from zope.interface import implementer
@@ -24,24 +24,24 @@ from twisted.conch.test.test_filetransfer import FileTransferTestAvatar, SFTPTes
 from twisted.cred import portal
 from twisted.internet import defer, error, interfaces, protocol, reactor
 from twisted.internet.task import Clock
-from twisted.internet.utils import getProcessOutputAndValue, getProcessValue
+from twisted.internet.testing import StringTransport
+from twisted.internet.utils import getProcessOutputAndValue
 from twisted.python import log
 from twisted.python.fakepwd import UserDatabase
 from twisted.python.filepath import FilePath
 from twisted.python.procutils import which
 from twisted.python.reflect import requireModule
-from twisted.test.proto_helpers import StringTransport
 from twisted.trial.unittest import TestCase
 
-pyasn1 = requireModule("pyasn1")
 cryptography = requireModule("cryptography")
 unix = requireModule("twisted.conch.unix")
 
-if cryptography and pyasn1:
+if cryptography:
     try:
         from twisted.conch.scripts import cftp
         from twisted.conch.scripts.cftp import SSHSession
         from twisted.conch.ssh import filetransfer
+        from twisted.conch.ssh.connection import EXTENDED_DATA_STDERR
         from twisted.conch.test import test_conch, test_ssh
         from twisted.conch.test.test_conch import FakeStdio
         from twisted.conch.test.test_filetransfer import FileTransferForTestAvatar
@@ -49,26 +49,48 @@ if cryptography and pyasn1:
         pass
 
 skipTests = False
-if None in (unix, cryptography, pyasn1, interfaces.IReactorProcess(reactor, None)):
+if None in (unix, cryptography, interfaces.IReactorProcess(reactor, None)):
     skipTests = True
 
 
-@skipIf(skipTests, "don't run w/o spawnProcess or cryptography or pyasn1")
+@skipIf(skipTests, "don't run w/o spawnProcess or cryptography")
 class SSHSessionTests(TestCase):
     """
     Tests for L{twisted.conch.scripts.cftp.SSHSession}.
     """
 
-    def test_eofReceived(self):
+    def setUp(self) -> None:
+        self.stdio = FakeStdio()
+        self.channel = SSHSession()
+        self.channel.stdio = self.stdio
+        self.stderrBuffer = BytesIO()
+        self.stderr = TextIOWrapper(self.stderrBuffer)
+        self.channel.stderr = self.stderr
+
+    def test_eofReceived(self) -> None:
         """
         L{twisted.conch.scripts.cftp.SSHSession.eofReceived} loses the write
         half of its stdio connection.
         """
-        stdio = FakeStdio()
-        channel = SSHSession()
-        channel.stdio = stdio
-        channel.eofReceived()
-        self.assertTrue(stdio.writeConnLost)
+        self.channel.eofReceived()
+        self.assertTrue(self.stdio.writeConnLost)
+
+    def test_extReceivedStderr(self) -> None:
+        """
+        L{twisted.conch.scripts.cftp.SSHSession.extReceived} decodes
+        stderr data using UTF-8 with the "backslashescape" error handling and
+        writes the result to its own stderr.
+        """
+        errorText = "\N{SNOWMAN}"
+        errorBytes = errorText.encode("utf-8")
+        self.channel.extReceived(
+            EXTENDED_DATA_STDERR,
+            errorBytes + b"\xff",
+        )
+        self.assertEqual(
+            self.stderrBuffer.getvalue(),
+            errorBytes + b"\\xff",
+        )
 
 
 class ListingTests(TestCase):
@@ -365,7 +387,7 @@ class InMemoryRemoteFile(BytesIO):
         return BytesIO.getvalue(self)
 
 
-@skipIf(skipTests, "don't run w/o spawnProcess or cryptography or pyasn1")
+@skipIf(skipTests, "don't run w/o spawnProcess or cryptography")
 class StdioClientTests(TestCase):
     """
     Tests for L{cftp.StdioClient}.
@@ -871,11 +893,11 @@ class SFTPTestProcess(protocol.ProcessProtocol):
 
 class CFTPClientTestBase(SFTPTestBase):
     def setUp(self):
-        with open("dsa_test.pub", "wb") as f:
-            f.write(test_ssh.publicDSA_openssh)
-        with open("dsa_test", "wb") as f:
-            f.write(test_ssh.privateDSA_openssh)
-        os.chmod("dsa_test", 33152)
+        with open("rsa_test.pub", "wb") as f:
+            f.write(test_ssh.publicRSA_openssh)
+        with open("rsa_test", "wb") as f:
+            f.write(test_ssh.privateRSA_openssh)
+        os.chmod("rsa_test", 33152)
         with open("kh_test", "wb") as f:
             f.write(b"127.0.0.1 " + test_ssh.publicRSA_openssh)
         return SFTPTestBase.setUp(self)
@@ -900,7 +922,7 @@ class CFTPClientTestBase(SFTPTestBase):
         return defer.maybeDeferred(self.server.stopListening)
 
     def tearDown(self):
-        for f in ["dsa_test.pub", "dsa_test", "kh_test"]:
+        for f in ["rsa_test.pub", "rsa_test", "kh_test"]:
             try:
                 os.remove(f)
             except BaseException:
@@ -908,7 +930,7 @@ class CFTPClientTestBase(SFTPTestBase):
         return SFTPTestBase.tearDown(self)
 
 
-@skipIf(skipTests, "don't run w/o spawnProcess or cryptography or pyasn1")
+@skipIf(skipTests, "don't run w/o spawnProcess or cryptography")
 class OurServerCmdLineClientTests(CFTPClientTestBase):
     """
     Functional tests which launch a SFTP server over TCP on localhost and check
@@ -927,7 +949,7 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
             "--known-hosts kh_test "
             "--user-authentications publickey "
             "--host-key-algorithms ssh-rsa "
-            "-i dsa_test "
+            "-i rsa_test "
             "-a "
             "-v "
             "127.0.0.1"
@@ -1307,7 +1329,7 @@ class OurServerCmdLineClientTests(CFTPClientTestBase):
         return d
 
 
-@skipIf(skipTests, "don't run w/o spawnProcess or cryptography or pyasn1")
+@skipIf(skipTests, "don't run w/o spawnProcess or cryptography")
 class OurServerBatchFileTests(CFTPClientTestBase):
     """
     Functional tests which launch a SFTP server over localhost and checks csftp
@@ -1332,7 +1354,7 @@ class OurServerBatchFileTests(CFTPClientTestBase):
             "--known-hosts kh_test "
             "--user-authentications publickey "
             "--host-key-algorithms ssh-rsa "
-            "-i dsa_test "
+            "-i rsa_test "
             "-a "
             "-v -b %s 127.0.0.1"
         ) % (port, fn)
@@ -1411,7 +1433,7 @@ exit
         return d
 
 
-@skipIf(skipTests, "don't run w/o spawnProcess or cryptography or pyasn1")
+@skipIf(skipTests, "don't run w/o spawnProcess or cryptography")
 @skipIf(not which("ssh"), "no ssh command-line client available")
 @skipIf(not which("sftp"), "no sftp command-line client available")
 class OurServerSftpClientTests(CFTPClientTestBase):
@@ -1453,36 +1475,26 @@ class OurServerSftpClientTests(CFTPClientTestBase):
         self.patch(FileTransferForTestAvatar, "_getAttrs", _getAttrs)
         self.server.factory.expectedLoseConnection = True
 
-        # PubkeyAcceptedKeyTypes does not exist prior to OpenSSH 7.0 so we
-        # first need to check if we can set it. If we can, -V will just print
-        # the version without doing anything else; if we can't, we will get a
-        # configuration error.
-        d = getProcessValue("ssh", ("-o", "PubkeyAcceptedKeyTypes=ssh-dss", "-V"), env)
-
-        def hasPAKT(status):
-            if status == 0:
-                args = ("-o", "PubkeyAcceptedKeyTypes=ssh-dss")
-            else:
-                args = ()
-            # Pass -F /dev/null to avoid the user's configuration file from
-            # being loaded, as it may contain settings that cause our tests to
-            # fail or hang.
-            args += (
-                "-F",
-                "/dev/null",
-                "-o",
-                "IdentityFile=dsa_test",
-                "-o",
-                "UserKnownHostsFile=kh_test",
-                "-o",
-                "HostKeyAlgorithms=ssh-rsa",
-                "-o",
-                "Port=%i" % (port,),
-                "-b",
-                fn,
-                "testuser@127.0.0.1",
-            )
-            return args
+        # Pass -F /dev/null to avoid the user's configuration file from
+        # being loaded, as it may contain settings that cause our tests to
+        # fail or hang.
+        args = (
+            "-F",
+            "/dev/null",
+            "-o",
+            "IdentitiesOnly=yes",
+            "-o",
+            "IdentityFile=rsa_test",
+            "-o",
+            "UserKnownHostsFile=kh_test",
+            "-o",
+            "HostKeyAlgorithms=ssh-rsa",
+            "-o",
+            "Port=%i" % (port,),
+            "-b",
+            fn,
+            "testuser@127.0.0.1",
+        )
 
         def check(result):
             self.assertEqual(result[2], 0, result[1].decode("ascii"))
@@ -1494,6 +1506,5 @@ class OurServerSftpClientTests(CFTPClientTestBase):
             ]:
                 self.assertIn(i, result[0])
 
-        d.addCallback(hasPAKT)
-        d.addCallback(lambda args: getProcessOutputAndValue("sftp", args, env))
+        d = getProcessOutputAndValue("sftp", args, env)
         return d.addCallback(check)
