@@ -24,7 +24,7 @@ from typing import (
     cast,
 )
 from xml.sax import handler, make_parser
-from xml.sax.xmlreader import Locator
+from xml.sax.xmlreader import AttributesNSImpl, Locator
 
 from zope.interface import implementer
 
@@ -92,7 +92,7 @@ def redirectTo(URL: bytes, request: IRequest) -> bytes:
     </body>
 </html>
 """ % {
-        b"url": URL
+        b"url": escape(URL.decode("utf-8")).encode("utf-8")
     }
     return content
 
@@ -118,34 +118,6 @@ class Redirect(resource.Resource):
         return self
 
 
-# FIXME: This is totally broken, see https://twistedmatrix.com/trac/ticket/9838
-class ChildRedirector(Redirect):
-    isLeaf = False
-
-    def __init__(self, url):
-        # XXX is this enough?
-        if (
-            (url.find("://") == -1)
-            and (not url.startswith(".."))
-            and (not url.startswith("/"))
-        ):
-            raise ValueError(
-                (
-                    "It seems you've given me a redirect (%s) that is a child of"
-                    " myself! That's not good, it'll cause an infinite redirect."
-                )
-                % url
-            )
-        Redirect.__init__(self, url)
-
-    def getChild(self, name, request):
-        newUrl = self.url
-        if not newUrl.endswith("/"):
-            newUrl += "/"
-        newUrl += name
-        return ChildRedirector(newUrl)
-
-
 class ParentRedirect(resource.Resource):
     """
     Redirect to the nearest directory and strip any query string.
@@ -269,7 +241,7 @@ class _SourceFragmentElement(Element):
         C{"snippetHighlightLine"}.  Other lines will be given a I{class} of
         C{"snippetLine"}.
         """
-        for (lineNumber, sourceLine) in self._getSourceLines():
+        for lineNumber, sourceLine in self._getSourceLines():
             newTag = tag.clone()
             if lineNumber == self.frame[2]:
                 cssClass = "snippetHighlightLine"
@@ -462,7 +434,7 @@ class _ToStan(handler.ContentHandler, handler.EntityResolver):
         self,
         namespaceAndName: Tuple[str, str],
         qname: Optional[str],
-        attrs: Mapping[Tuple[Optional[str], str], str],
+        attrs: AttributesNSImpl,
     ) -> None:
         """
         Gets called when we encounter a new xmlns attribute.
@@ -505,14 +477,14 @@ class _ToStan(handler.ContentHandler, handler.EntityResolver):
 
         render = None
 
-        attrs = OrderedDict(attrs)
-        for k, v in list(attrs.items()):
+        ordered = OrderedDict(attrs)
+        for k, v in list(ordered.items()):
             attrNS, justTheName = k
             if attrNS != TEMPLATE_NAMESPACE:
                 continue
             if justTheName == "render":
                 render = v
-                del attrs[k]
+                del ordered[k]
 
         # nonTemplateAttrs is a dictionary mapping attributes that are *not* in
         # TEMPLATE_NAMESPACE to their values.  Those in TEMPLATE_NAMESPACE were
@@ -522,7 +494,7 @@ class _ToStan(handler.ContentHandler, handler.EntityResolver):
         # preserving the xml namespace prefix given in the document.
 
         nonTemplateAttrs = OrderedDict()
-        for (attrNs, attrName), v in attrs.items():
+        for (attrNs, attrName), v in ordered.items():
             nsPrefix = self.prefixMap.get(attrNs)
             if nsPrefix is None:
                 attrKey = attrName
@@ -856,7 +828,7 @@ class XMLFile:
     An L{ITemplateLoader} that loads and parses XML from a file.
     """
 
-    def __init__(self, path: FilePath):
+    def __init__(self, path: FilePath[Any]):
         """
         Run the parser on a file.
 
@@ -873,7 +845,7 @@ class XMLFile:
         self._loadedTemplate: Optional[List["Flattenable"]] = None
         """The loaded document, or L{None}, if not loaded."""
 
-        self._path: FilePath = path
+        self._path: FilePath[Any] = path
         """The file that is being loaded from."""
 
     def _loadDoc(self) -> List["Flattenable"]:
@@ -1034,9 +1006,9 @@ class _TagFactory:
     """
     A factory for L{Tag} objects; the implementation of the L{tags} object.
 
-    This allows for the syntactic convenience of C{from twisted.web.html import
-    tags; tags.a(href="linked-page.html")}, where 'a' can be basically any HTML
-    tag.
+    This allows for the syntactic convenience of C{from twisted.web.template
+    import tags; tags.a(href="linked-page.html")}, where 'a' can be basically
+    any HTML tag.
 
     The class is not exposed publicly because you only ever need one of these,
     and we already made it for you.
